﻿/****************************************************************
 * @class CommunicationCustomVocabBuilderPage
 * 
 * @author Imran Merchant
 * 
 * Adds vocabulary options to categories.
 * 
 ***************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using PowerDroid.TextToSpeech;
using System.IO.IsolatedStorage;
using System.IO;

namespace PowerDroid
{
	public partial class CommunicationCustomVocabPage : PhoneApplicationPage
	{
		private LanguageServiceClient LanguageClient = new LanguageServiceClient();
		private WebClient WClient = new WebClient();
		private string ToSpeak = "";
		private string cat = "";
		private string alpha = (Application.Current as App).alphabet;
		private PhotoChooserTask ChoosePhotoTask;
		private string phrase = "";
		public CommunicationCustomVocabPage()
		{
			InitializeComponent();
			powerAppBar1.parent = this;

			LanguageClient.SpeakCompleted += SpeakCompleted;
			WClient.OpenReadCompleted += WCOpenReadCompleted;

			ChoosePhotoTask = new PhotoChooserTask();
			ChoosePhotoTask.Completed += new EventHandler<PhotoResult>(ChoosePhotoTask_Completed);


		}

		void ChoosePhotoTask_Completed(object sender, PhotoResult e)
		{
			if(e.TaskResult == TaskResult.OK && e.ChosenPhoto != null)
			{
				BitmapImage image = new BitmapImage();
				image.SetSource(e.ChosenPhoto);

				PreviewImage.Source = image;
			}
			PhraseBox.Dispatcher.BeginInvoke(() =>
				{
					PhraseBox.Text = phrase;
				});
		}

		protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
		{
			IDictionary<string, string> parameters = this.NavigationContext.QueryString;
			if(parameters.ContainsKey("category"))
			{
				switch(Int32.Parse(parameters["category"]))
				{
					case 0: cat = "Greetings"; break;
					case 1: cat = "People"; break;
					case 2: cat = "Places"; break;
					case 3: cat = "Food"; break;
					case 4: cat = "Questions"; break;
					case 5: cat = "Medical"; break;
				}
				ApplicationTitle.Dispatcher.BeginInvoke(() =>
					{
						if(!PageTitle.Text.Contains(cat))
							PageTitle.Text += cat;
					});
				
			}
			if(parameters.ContainsKey("phrase"))
			{
				PhraseBox.Dispatcher.BeginInvoke(() =>
					{
						PhraseBox.Text = parameters["phrase"];
					});
			}

			(Application.Current as App).AddVocabFired = true;
			
			base.OnNavigatedTo(e);
		}

		private void SelectImageButton_Click(object sender, RoutedEventArgs e)
		{
			phrase = PhraseBox.Text;
			ChoosePhotoTask.Show();
		}

		private void PhraseBox_TextChanged(object sender, TextChangedEventArgs e)
		{
			// I'm not entirely sure this is needed, but used it to err on the side of caution
			PreviewTextBlock.Dispatcher.BeginInvoke(() =>
			{
				PreviewTextBlock.Text = PhraseBox.Text;
			});
		}

		private void ListenButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(PreviewTextBlock.Text);
		}

		private void SaveButton_Click(object sender, RoutedEventArgs e)
		{
			CustomVocab customVocab = new CustomVocab(PhraseBox.Text, PreviewImage);
			App ThisApp = Application.Current as App;
			switch(cat)
			{
				case "Greetings": ThisApp.GreetingsCustom.Add(customVocab); break;
				case "People": ThisApp.PeopleCustom.Add(customVocab); break;
				case "Places": ThisApp.PlacesCustom.Add(customVocab); break;
				case "Food": ThisApp.FoodCustom.Add(customVocab); break;
				case "Questions": ThisApp.QuestionsCustom.Add(customVocab); break;
				case "Medical": ThisApp.MedicalCustom.Add(customVocab); break;
			}
			this.NavigationService.GoBack();
		}

		private void PlaySentence(string sentence)
		{
			// need to sanitize sentence, can't have punctuation making it into the file system
			ToSpeak = new string(sentence.ToCharArray());
			for(int k = 0; k < ToSpeak.Length; k++)
			{
				if(!alpha.Contains(ToSpeak.ElementAt(k))) // non-alpha character detected
				{
					ToSpeak = ToSpeak.Replace(ToSpeak.ElementAt(k)+"",""); // remove all instances of that character
					k = -1; // restart from beginning of sentence
				}
			}
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech_" + ToSpeak; // cache created with phrase as part of file name
			bool exists = false; // repeated calls to the same phrase don't result in more network calls
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
				{
					exists = true; // pulls file from cache

					var FileStream = userStoreForApplication.OpenFile(temp, FileMode.Open);

					TextToSpeechMediaElement.SetSource(FileStream);
					TextToSpeechMediaElement.Play();
				}

			}
			if(!exists) LanguageClient.SpeakAsync("A05A61388A448F55C9628C941FB2FC459D697060", sentence, "en", String.Empty);
		}
		void SpeakCompleted(object sender, SpeakCompletedEventArgs e)
		{
			if(e.Error != null)
				return;

			WClient.OpenReadAsync(new Uri(e.Result, UriKind.Absolute));
		}
		void WCOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
		{
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech_" + ToSpeak;
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
				{
					userStoreForApplication.DeleteFile(temp);
				}

				var storageFileStream = userStoreForApplication.CreateFile(temp);

				using(storageFileStream)
				{
					CopyStream(e.Result, storageFileStream);
					if(e.Error == null)
					{
						TextToSpeechMediaElement.SetSource(storageFileStream);
						TextToSpeechMediaElement.Play();
					}
				}
			}
		}
		private void CopyStream(Stream input, Stream output)
		{
			byte[] buffer = new byte[32768];
			while(true)
			{
				int read = input.Read(buffer, 0, buffer.Length);
				if(read <= 0)
					return;
				output.Write(buffer, 0, buffer.Length);
			}
		}
	}
}