﻿/****************************************************************
 * @class CommunicationsListingPage
 * 
 * @author Imran Merchant
 * 
 * Favorite/common/recent sentence chooser
 * 
 ***************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Markup;
using System.Windows.Shapes;
using PowerDroid.TextToSpeech;
using System.IO.IsolatedStorage;
using System.IO;
using Microsoft.Phone.Controls;

namespace PowerDroid
{
	public partial class CommunicationListingPage : PhoneApplicationPage
	{
		private List<string> favorites = (Application.Current as App).FavoritesList;
		private List<string> common = (Application.Current as App).CommonList;
		private List<string> recent = (Application.Current as App).RecentList;
		private LanguageServiceClient LanguageClient = new LanguageServiceClient();
		private WebClient WClient = new WebClient();
		private string ToSpeak = "";
		private string alpha = (Application.Current as App).alphabet;
		public CommunicationListingPage()
		{
			InitializeComponent();

			powerAppBar1.parent = this;

			LanguageClient.SpeakCompleted += SpeakCompleted;
			WClient.OpenReadCompleted += WCOpenReadCompleted;

		/*	favorites.Add("Hello my name is Herbert");
			favorites.Add("Where is the bathroom");
			favorites.Add("We need to go pick up my sister from the mall");
			favorites.Add("I want a Coke");
			favorites.Add("Where are we going");*/

			FillFavoritesStackPanel();
			FillCommonStackPanel();
			FillRecentStackPanel();
		}
		protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
		{
			// Just in case, since we /are/ modifying the UI
			CommPivot.Dispatcher.BeginInvoke(() => // This still bugs me. new dispatch() is an alternative. Still confusing.
				{
					FillCommonStackPanel();
					FillRecentStackPanel();
				});
			base.OnNavigatedTo(e);
		}
		private void FillFavoritesStackPanel()
		{
			if(favorites.Count >= 1)
			{
				FirstFavGrid.Visibility = System.Windows.Visibility.Visible;
				FirstFavTextBlock.Text = favorites.ElementAt(0);
			}
			if(favorites.Count >= 2) 
			{
				SecondFavGrid.Visibility = System.Windows.Visibility.Visible;
				SecondFavTextBlock.Text = favorites.ElementAt(1);
			}
			if(favorites.Count >= 3)
			{
				ThirdFavGrid.Visibility = System.Windows.Visibility.Visible;
				ThirdFavTextBlock.Text = favorites.ElementAt(2);
			}
			if(favorites.Count >= 4)
			{
				FourthFavGrid.Visibility = System.Windows.Visibility.Visible;
				FourthFavTextBlock.Text = favorites.ElementAt(3);
			}
			if(favorites.Count >= 5)
			{
				FifthFavGrid.Visibility = System.Windows.Visibility.Visible;
				FifthFavTextBlock.Text = favorites.ElementAt(4);
			}
		}
		void SpeakCompleted(object sender, SpeakCompletedEventArgs e)
		{
			if(e.Error != null)
				return;

			WClient.OpenReadAsync(new Uri(e.Result, UriKind.Absolute));
		}
		void WCOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
		{
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech";
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
					userStoreForApplication.DeleteFile(temp);

				var storageFileStream = userStoreForApplication.CreateFile(temp);

				using(storageFileStream)
				{
					CopyStream(e.Result, storageFileStream);
					if(e.Error == null)
					{
						TextToSpeechMediaElement.SetSource(storageFileStream);
						TextToSpeechMediaElement.Play();
					}
				}
			}
		}
		private void CopyStream(Stream input, Stream output)
		{
			byte[] buffer = new byte[32768];
			while(true)
			{
				int read = input.Read(buffer, 0, buffer.Length);
				if(read <= 0)
					return;
				output.Write(buffer, 0, buffer.Length);
			}
		}
		private void PlaySentence1Button_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(FirstFavTextBlock.Text);
		}
		private void PlaySentence(string sentence)
		{
			// need to sanitize sentence, can't have punctuation making it into the file system
			ToSpeak = new string(sentence.ToCharArray());
			for(int k = 0; k < ToSpeak.Length; k++)
			{
				if(!alpha.Contains(ToSpeak.ElementAt(k))) // non-alpha character detected
				{
					ToSpeak = ToSpeak.Replace(ToSpeak.ElementAt(k) + "", ""); // remove all instances of that character
					k = -1; // restart from beginning of sentence
				}
			}
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech_" + ToSpeak; // cache created with phrase as part of file name
			bool exists = false; // repeated calls to the same phrase don't result in more network calls
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
				{
					exists = true; // pulls file from cache

					var FileStream = userStoreForApplication.OpenFile(temp, FileMode.Open);

					TextToSpeechMediaElement.SetSource(FileStream);
					TextToSpeechMediaElement.Play();
				}

			}
			if(!exists) LanguageClient.SpeakAsync("A05A61388A448F55C9628C941FB2FC459D697060", sentence, "en", String.Empty);
		}

		private void PlaySentence2Button_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(SecondFavTextBlock.Text);
		}

		private void PlaySentence3Button_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(ThirdFavTextBlock.Text);
		}

		private void PlaySentence4Button_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(FourthFavTextBlock.Text);
		}

		private void PlaySentence5Button_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(FifthFavTextBlock.Text);
		}

		private void NewSentence_Click(object sender, RoutedEventArgs e)
		{
			NavigationService.Navigate(new Uri("/CommunicationCategoriesPage.xaml", UriKind.Relative));
		}

		private void FillRecentStackPanel()
		{
			if(recent.Count == 0) return;
			for(int k = 0; k < recent.Count; k++)
			{
				object textblock = this.FindName("Recent" + (k + 1) + "Text");
				(textblock as TextBlock).Text = recent[k];
				object grid = this.FindName("Recent" + (k + 1) + "Grid");
				(grid as Grid).Visibility = System.Windows.Visibility.Visible;
				if(k == 9) break;
			}
		}

		private void FillCommonStackPanel()
		{
			if(common.Count == 0) return;
			for(int k = 0; k < common.Count; k++)
			{
				object textblock = this.FindName("Common" + (k + 1) + "Text");
				(textblock as TextBlock).Text = common[k];
				object grid = this.FindName("Common" + (k + 1) + "Grid");
				(grid as Grid).Visibility = System.Windows.Visibility.Visible;
				if(k == 9) break;
			}
		}

		private void Recent1PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent1Text.Text);
		}

		private void Recent2PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent2Text.Text);
		}

		private void Recent3PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent3Text.Text);
		}

		private void Recent4PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent4Text.Text);
		}

		private void Recent5PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent5Text.Text);
		}

		private void Recent6PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent6Text.Text);
		}

		private void Recent7PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent7Text.Text);
		}

		private void Recent8PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent8Text.Text);
		}

		private void Recent9PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent9Text.Text);
		}

		private void Recent10PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Recent10Text.Text);
		}
		private void Common1PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common1Text.Text);
		}

		private void Common2PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common2Text.Text);
		}

		private void Common3PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common3Text.Text);
		}

		private void Common4PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common4Text.Text);
		}

		private void Common5PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common5Text.Text);
		}

		private void Common6PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common6Text.Text);
		}

		private void Common7PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common7Text.Text);
		}

		private void Common8PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common8Text.Text);
		}

		private void Common9PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common9Text.Text);
		}

		private void Common10PlayButton_Click(object sender, RoutedEventArgs e)
		{
			PlaySentence(Common10Text.Text);
		}

		private void DeleteSentence1Button_Click(object sender, RoutedEventArgs e)
		{
			favorites.Remove(FirstFavTextBlock.Text);
			FirstFavGrid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void DeleteSentence2Button_Click(object sender, RoutedEventArgs e)
		{
			favorites.Remove(SecondFavTextBlock.Text);
			SecondFavGrid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void DeleteSentence3Button_Click(object sender, RoutedEventArgs e)
		{
			favorites.Remove(ThirdFavTextBlock.Text);
			ThirdFavGrid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void DeleteSentence4Button_Click(object sender, RoutedEventArgs e)
		{
			favorites.Remove(FourthFavTextBlock.Text);
			FourthFavGrid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void DeleteSentence5Button_Click(object sender, RoutedEventArgs e)
		{
			favorites.Remove(FifthFavTextBlock.Text);
			FifthFavGrid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common1Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common1Text.Text);
			Common1Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common2Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common2Text.Text);
			Common2Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common3Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common3Text.Text);
			Common3Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common4Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common4Text.Text);
			Common4Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common5Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common5Text.Text);
			Common5Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common6Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common6Text.Text);
			Common6Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common7Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common7Text.Text);
			Common7Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common8Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common8Text.Text);
			Common8Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common9Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common9Text.Text);
			Common9Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Common10Delete_Click(object sender, RoutedEventArgs e)
		{
			common.Remove(Common10Text.Text);
			Common10Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent1Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent1Text.Text);
			Recent1Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent2Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent2Text.Text);
			Recent2Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent3Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent3Text.Text);
			Recent3Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent4Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent4Text.Text);
			Recent4Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent5Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent5Text.Text);
			Recent5Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent6Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent6Text.Text);
			Recent6Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent7Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent7Text.Text);
			Recent7Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent8Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent8Text.Text);
			Recent8Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent9Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent9Text.Text);
			Recent9Grid.Visibility = System.Windows.Visibility.Collapsed;
		}

		private void Recent10Delete_Click(object sender, RoutedEventArgs e)
		{
			recent.Remove(Recent10Text.Text);
			Recent10Grid.Visibility = System.Windows.Visibility.Collapsed;
		}
	}
}