﻿/****************************************************************
 * @class CommunicationsSentenceBuilderPage
 * 
 * @author Imran Merchant
 * 
 * Builds sentences, categories are on Pivots
 * 
 ***************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using PowerDroid.TextToSpeech;
using System.IO.IsolatedStorage;
using System.IO;


namespace PowerDroid
{
	public partial class CommunicationsSentenceBuilderPage : PhoneApplicationPage
	{
		private LanguageServiceClient LanguageClient = new LanguageServiceClient();
		private WebClient WClient = new WebClient();
		private string ToSpeak = "";
		private List<string> favList; // Used for local reference
		private List<string> commList; // Used for local reference
		private List<string> recentList; // Used for local reference
		private string alpha = (Application.Current as App).alphabet; // Used for local reference to sanitize T2S cache
		private int CustomsAvailableGreetings = 4,	// Number of XAML constructs available for custom vocab. Very hacky.
					CustomsAvailablePeople = 3,
					CustomsAvailablePlaces = 4,
					CustomsAvailableFood = 5,
					CustomsAvailableQuestions = 5,
					CustomsAvailableMedical = 4;
		private string[] Categories = { "Greetings", "People", "Places", "Food", "Questions", "Medical" };
		public CommunicationsSentenceBuilderPage()
		{
			InitializeComponent();

			powerAppBar1.parent = this;
			favList = (Application.Current as App).FavoritesList;
			commList = (Application.Current as App).CommonList;
			recentList = (Application.Current as App).RecentList;

			LanguageClient.SpeakCompleted += SpeakCompleted;
			WClient.OpenReadCompleted += WCOpenReadCompleted;

			FillCustoms();
		}

		// This is _very_ hacky, but short of re-architecting the entire page, this is the
		// fastest way to get basic customization working. Not happy with this at all.
		private void FillCustoms()
		{
			App ThisApp = Application.Current as App;

			for(int k = 0; k < ThisApp.GreetingsCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailableGreetings) break;
				(this.FindName("CustomGreetings" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomGreetingsImage" + (k + 1)) as Image).Source = ThisApp.GreetingsCustom[k].DisplayImage.Source;
				(this.FindName("CustomGreetingsDisplay" + (k + 1)) as TextBlock).Text = ThisApp.GreetingsCustom[k].DisplayPhrase;
				(this.FindName("CustomGreetingsPhrase" + (k + 1)) as TextBlock).Text = ThisApp.GreetingsCustom[k].Phrase + " ";
			}

			for(int k = 0; k < ThisApp.PeopleCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailablePeople) break;
				(this.FindName("CustomPeople" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomPeopleImage" + (k + 1)) as Image).Source = ThisApp.PeopleCustom[k].DisplayImage.Source;
				(this.FindName("CustomPeopleDisplay" + (k + 1)) as TextBlock).Text = ThisApp.PeopleCustom[k].DisplayPhrase;
				(this.FindName("CustomPeoplePhrase" + (k + 1)) as TextBlock).Text = ThisApp.PeopleCustom[k].Phrase + " ";
			}

			for(int k = 0; k < ThisApp.PlacesCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailablePlaces) break;
				(this.FindName("CustomPlaces" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomPlacesImage" + (k + 1)) as Image).Source = ThisApp.PlacesCustom[k].DisplayImage.Source;
				(this.FindName("CustomPlacesDisplay" + (k + 1)) as TextBlock).Text = ThisApp.PlacesCustom[k].DisplayPhrase;
				(this.FindName("CustomPlacesPhrase" + (k + 1)) as TextBlock).Text = ThisApp.PlacesCustom[k].Phrase + " ";
			}

			for(int k = 0; k < ThisApp.FoodCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailableFood) break;
				(this.FindName("CustomFood" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomFoodImage" + (k + 1)) as Image).Source = ThisApp.FoodCustom[k].DisplayImage.Source;
				(this.FindName("CustomFoodDisplay" + (k + 1)) as TextBlock).Text = ThisApp.FoodCustom[k].DisplayPhrase;
				(this.FindName("CustomFoodPhrase" + (k + 1)) as TextBlock).Text = ThisApp.FoodCustom[k].Phrase + " ";
			}

			for(int k = 0; k < ThisApp.QuestionsCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailableQuestions) break;
				(this.FindName("CustomQuestions" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomQuestionsImage" + (k + 1)) as Image).Source = ThisApp.QuestionsCustom[k].DisplayImage.Source;
				(this.FindName("CustomQuestionsDisplay" + (k + 1)) as TextBlock).Text = ThisApp.QuestionsCustom[k].DisplayPhrase;
				(this.FindName("CustomQuestionsPhrase" + (k + 1)) as TextBlock).Text = ThisApp.QuestionsCustom[k].Phrase + " ";
			}

			for(int k = 0; k < ThisApp.MedicalCustom.Count; k++)
			{
				if(k + 1 > CustomsAvailableMedical) break;
				(this.FindName("CustomMedical" + (k + 1)) as Button).Visibility = System.Windows.Visibility.Visible;
				(this.FindName("CustomMedicalImage" + (k + 1)) as Image).Source = ThisApp.MedicalCustom[k].DisplayImage.Source;
				(this.FindName("CustomMedicalDisplay" + (k + 1)) as TextBlock).Text = ThisApp.MedicalCustom[k].DisplayPhrase;
				(this.FindName("CustomMedicalPhrase" + (k + 1)) as TextBlock).Text = ThisApp.MedicalCustom[k].Phrase + " ";
			}
		}

		private void ClearSentence_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text = "";
		}

		private void PlaySentence_Click(object sender, RoutedEventArgs e)
		{
			PlayString(SentenceBox.Text);
		}
		private void PlayString(string sentence)
		{
			// need to sanitize sentence, can't have punctuation making it into the file system
			ToSpeak = new string(sentence.ToCharArray());
			for(int k = 0; k < ToSpeak.Length; k++)
			{
				if(!alpha.Contains(ToSpeak.ElementAt(k))) // non-alpha character detected
				{
					ToSpeak = ToSpeak.Replace(ToSpeak.ElementAt(k) + "", ""); // remove all instances of that character
					k = -1; // restart from beginning of sentence
				}
			}
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech_" + ToSpeak; // cache created with phrase as part of file name
			bool exists = false; // repeated calls to the same phrase don't result in more network calls
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
				{
					exists = true; // pulls file from cache

					var FileStream = userStoreForApplication.OpenFile(temp, FileMode.Open);

					TextToSpeechMediaElement.SetSource(FileStream);
					TextToSpeechMediaElement.Play();

					
				}
				
			}
			if(!exists) LanguageClient.SpeakAsync("A05A61388A448F55C9628C941FB2FC459D697060", sentence, "en", String.Empty);
			if(!recentList.Contains(sentence))
			{
				recentList.Insert(0, sentence);
			}
			if(!commList.Contains(sentence))
			{
				commList.Add(sentence);
			}
			else
			{
				commList.Remove(sentence);
				commList.Insert(0, sentence);
			}
		}
		void SpeakCompleted(object sender, SpeakCompletedEventArgs e)
		{
			if(e.Error != null)
				return;

			WClient.OpenReadAsync(new Uri(e.Result, UriKind.Absolute));
		}
		void WCOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
		{
			TextToSpeechMediaElement.Source = null;
			string temp = "tempTextToSpeech_"+ToSpeak;
			using(IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if(userStoreForApplication.FileExists(temp))
				{
					userStoreForApplication.DeleteFile(temp);
				}

				var storageFileStream = userStoreForApplication.CreateFile(temp);

				using(storageFileStream)
				{
					CopyStream(e.Result, storageFileStream);
					if(e.Error == null)
					{
						TextToSpeechMediaElement.SetSource(storageFileStream);
						TextToSpeechMediaElement.Play();
					}
				}
			}
		}
		private void CopyStream(Stream input, Stream output)
		{
			byte[] buffer = new byte[32768];
			while(true)
			{
				int read = input.Read(buffer, 0, buffer.Length);
				if(read <= 0)
					return;
				output.Write(buffer, 0, buffer.Length);
			}
		}
		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			if((Application.Current as App).AddVocabFired)
			{
				(Application.Current as App).AddVocabFired = false;
				CatPivot.Dispatcher.BeginInvoke(() =>
					{
						FillCustoms();
					});
				base.OnNavigatedTo(e);
				return;
			}
			IDictionary<string,string> parameters = this.NavigationContext.QueryString;
			if(parameters.ContainsKey("category"))
			{
				System.Diagnostics.Debug.WriteLine(parameters["category"]);
				int target = 0;
				int source = CatPivot.SelectedIndex;
				switch(parameters["category"])
				{
					case "Greetings":
						target = 0;
						break;
					case "People":
						target = 1;
						break;
					case "Places":
						target = 2;
						break;
					case "Food":
						target = 3;
						break;
					case "Questions":
						target = 4;
						break;
					case "Medical":
						target = 5;
						break;
					default: target = 0;
						break;
				}
				 /* Need to call the CatPivot.SelectedIndex _on_ the UI Thread, after the UI has a chance to
				  * finish building itself. The Dispatcher waits for CatPivot to finish rendering
				  * before it changes the active Pivot. The () => syntax confuses the hell out of me.
				  * It appears the OnNavigatedTo() operates off the UI thread, so any UI changes need
				  * to be piped onto that thread. The Dispatcher does a great job of that.
				  * */
				CatPivot.Dispatcher.BeginInvoke(() =>
					{
						CatPivot.SelectedIndex = target;
					});
			/*	do
				{
					if(target == source || target == source + 1 || target == source - 1)
					{
						CatPivot.SelectedIndex = target;
						break;
					}
					else
					{
						source++;
						if(source == 6) source = 0;
						CatPivot.SelectedIndex = source;
					}
				} while(target != source);*/
				
			}

			CatPivot.Dispatcher.BeginInvoke(() =>
				{
					FillCustoms();
				});

			base.OnNavigatedTo(e);
		}

		private void HelloButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Hello ";
			PlayString("Hello");
		}

		private void GoodbyeButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Goodbye ";
			PlayString("Goodbye");
		}

		private void MyNameIsButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "My name is ";
			PlayString("My name is");
		}

		private void NiceToButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Nice to meet you ";
			PlayString("Nice to meet you");
		}

		private void HowAreYouButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "How are you ";
			PlayString("How are you");
		}

		private void PleaseButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Please ";
			PlayString("Please");
		}

		private void ThankYouButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Thank you ";
			PlayString("Thank you");
		}

		private void YesButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Yes ";
			PlayString("Yes");
		}

		private void NoButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "No ";
			PlayString("No");
		}

		private void MaybeButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Maybe ";
			PlayString("Maybe");
		}

		private void YoureWelcomeButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "You're welcome ";
			PlayString("You're welcome");
		}

		private void SentenceBox_KeyUp(object sender, KeyEventArgs e)
		{
			if(e.Key == System.Windows.Input.Key.Enter || e.PlatformKeyCode == 0x0A)
			{
				PlayString(SentenceBox.Text);
			}
		}

		private void FamilyButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Family ";
			PlayString("Family");
		}

		private void FluffersButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Fluffers ";
			PlayString("Fluffers");
		}

		private void CatAttackButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Steve ";
			PlayString("Steve");
		}

		private void HomeButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Home ";
			PlayString("Home");
		}

		private void WantToGoButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "I want to go to ";
			PlayString("I want to go to");
		}

		private void SleepButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Sleep ";
			PlayString("Sleep");
		}

		private void EatButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Eat ";
			PlayString("Eat");
		}

		private void RestroomButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Restroom ";
			PlayString("Restroom");
		}

		private void BatheButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Take a bath ";
			PlayString("Take a bath");
		}

		private void MedicalButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Doctor ";
			PlayString("Doctor");
		}

		private void MoonButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "The Moon ";
			PlayString("The Moon");
		}

		private void RestaurantButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Restaurant ";
			PlayString("Restaurant");
		}

		private void WhatButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "What ";
			PlayString("What");
		}

		private void WhoButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Who ";
			PlayString("Who");
		}

		private void WhenButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "When ";
			PlayString("When");
		}

		private void WhereButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "Where ";
			PlayString("Where");
		}

		private void HelpButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "I need help ";
			PlayString("I need help");
		}

		private void GoToDocButton_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += "I need to go to the doctor ";
			PlayString("I need to go to the doctor");
		}

		private void AddPhrase_Click(object sender, RoutedEventArgs e)
		{
			// This brings the information of which category to add the phrase to, and,
			// for user convenience, brings in the current phrase in the TextBox as a
			// starting point.
			TextToSpeechMediaElement.Source = null;
			string commands = "?category=" + CatPivot.SelectedIndex + "&phrase=" + SentenceBox.Text;
			NavigationService.Navigate(new Uri("/CommunicationCustomVocabPage.xaml"+commands, UriKind.Relative));
		}

		private void CustomGreetings1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomGreetingsPhrase1.Text;
			PlayString(CustomGreetingsPhrase1.Text);
		}

		private void CustomPeople1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPeoplePhrase1.Text;
			PlayString(CustomPeoplePhrase1.Text);
		}

		private void CustomPeople2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPeoplePhrase2.Text;
			PlayString(CustomPeoplePhrase2.Text);
		}

		private void CustomPlaces1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPlacesPhrase1.Text;
			PlayString(CustomPlacesPhrase1.Text);
		}

		private void CustomFood1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomFoodPhrase1.Text;
			PlayString(CustomFoodPhrase1.Text);
		}

		private void CustomQuestions1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomQuestionsPhrase1.Text;
			PlayString(CustomQuestionsPhrase1.Text);
		}

		private void CustomMedical1_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomMedicalPhrase1.Text;
			PlayString(CustomMedicalPhrase1.Text);
		}

		private void CustomGreetings2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomGreetingsPhrase2.Text;
			PlayString(CustomGreetingsPhrase2.Text);
		}

		private void CustomGreetings3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomGreetingsPhrase3.Text;
			PlayString(CustomGreetingsPhrase3.Text);
		}

		private void CustomGreetings4_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomGreetingsPhrase4.Text;
			PlayString(CustomGreetingsPhrase4.Text);
		}

		private void CustomPlaces2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPlacesPhrase2.Text;
			PlayString(CustomPlacesPhrase2.Text);
		}

		private void CustomPeople3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPeoplePhrase3.Text;
			PlayString(CustomPeoplePhrase3.Text);
		}

		private void CustomPlaces3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPlacesPhrase3.Text;
			PlayString(CustomPlacesPhrase3.Text);
		}

		private void CustomPlaces4_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomPlacesPhrase4.Text;
			PlayString(CustomPlacesPhrase4.Text);
		}

		private void CustomFood2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomFoodPhrase2.Text;
			PlayString(CustomFoodPhrase2.Text);
		}

		private void CustomFood3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomFoodPhrase3.Text;
			PlayString(CustomFoodPhrase3.Text);
		}

		private void CustomFood4_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomFoodPhrase4.Text;
			PlayString(CustomFoodPhrase4.Text);
		}

		private void CustomFood5_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomFoodPhrase5.Text;
			PlayString(CustomFoodPhrase5.Text);
		}

		private void CustomQuestions2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomQuestionsPhrase2.Text;
			PlayString(CustomQuestionsPhrase2.Text);
		}

		private void CustomQuestions3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomQuestionsPhrase3.Text;
			PlayString(CustomQuestionsPhrase3.Text);
		}

		private void CustomQuestions4_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomQuestionsPhrase4.Text;
			PlayString(CustomQuestionsPhrase4.Text);
		}

		private void CustomQuestions5_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomQuestionsPhrase5.Text;
			PlayString(CustomQuestionsPhrase5.Text);
		}

		private void CustomMedical2_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomMedicalPhrase2.Text;
			PlayString(CustomMedicalPhrase2.Text);
		}

		private void CustomMedical3_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomMedicalPhrase3.Text;
			PlayString(CustomMedicalPhrase3.Text);
		}

		private void CustomMedical4_Click(object sender, RoutedEventArgs e)
		{
			SentenceBox.Text += CustomMedicalPhrase4.Text;
			PlayString(CustomMedicalPhrase4.Text);
		}
	}
}