﻿/****************************************************************
 * @class CustomizationPage
 * 
 * @author Tommy Omen
 * 
 * Allows user to play voice communication
 * 
 ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using PowerDroid.TextToSpeech;
using System.IO;
using System.IO.IsolatedStorage;

namespace PowerDroid
{
    public partial class CustomizationPage : PhoneApplicationPage
    {
        private PhotoChooserTask m_photoChooserTask;
        private String m_description = "";

        public CustomizationPage()
        {
            InitializeComponent();
            m_photoChooserTask = new PhotoChooserTask();
            m_photoChooserTask.Completed += new EventHandler<PhotoResult>(m_photoChooserTask_Completed);
        }

        void client_SpeakCompleted(object sender, SpeakCompletedEventArgs e)
        {
            if (e.Error != null)
                return;

            WebClient wc = new WebClient();
            wc.OpenReadCompleted += WcOpenReadCompleted;
            wc.OpenReadAsync(new Uri(e.Result, UriKind.Absolute));
        }

        void WcOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            m_textToSpeechMediaElement.Source = null;
            string filename = "test";
            using (IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
            {
                bool fileExists = userStoreForApplication.FileExists(filename);

                if (fileExists)
                {
                    userStoreForApplication.DeleteFile(filename);
                }

                var isolatedStorageFileStream = userStoreForApplication.CreateFile(filename);

                using (isolatedStorageFileStream)
                {
                    CopyStream(e.Result, isolatedStorageFileStream);

                    if (e.Error == null)
                    {
                        m_textToSpeechMediaElement.SetSource(isolatedStorageFileStream);
                        m_textToSpeechMediaElement.Play();

                    }
                }
            }
        }

        public static void CopyStream(Stream input, Stream output)
        {
            byte[] buffer = new byte[32768];
            while (true)
            {
                int read = input.Read(buffer, 0, buffer.Length);
                if (read <= 0)
                {
                    return;
                }
                output.Write(buffer, 0, read);
            }
        }

        private void m_selectImageBtn_Click(object sender, RoutedEventArgs e)
        {
            m_photoChooserTask.Show();
        }

        private void m_descriptionTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            m_description = m_descriptionTextBox.Text;
        }

        private void m_previewBtn_Click(object sender, RoutedEventArgs e)
        {
            if (m_descriptionTextBox.Text.Equals("<Enter Description>"))
            {
                return;
            }

            LanguageServiceClient client = new LanguageServiceClient();
            client.SpeakCompleted += client_SpeakCompleted;
            client.SpeakAsync("A05A61388A448F55C9628C941FB2FC459D697060", m_description, "en", string.Empty);
        }

        private void m_photoChooserTask_Completed(object sender, PhotoResult e)
        {
            if (e.TaskResult == TaskResult.OK && e.ChosenPhoto != null )
            {
                //grab selected image from photo chooser task
                System.Windows.Media.Imaging.BitmapImage bmp =
                  new System.Windows.Media.Imaging.BitmapImage();
                bmp.SetSource(e.ChosenPhoto);

                //set the background image for the button
                ImageBrush imgBrush = new ImageBrush();
                imgBrush.ImageSource = bmp;
                m_previewBtn.Background = imgBrush;
                //remove text from button.
                m_previewBtn.Content = "";
            }
        }

        private void m_descriptionTextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            if (m_descriptionTextBox.Text.Equals("<Enter Description>"))
            {
                m_descriptionTextBox.Text = "";
            }
        }

        private void m_selectImageBtn_GotFocus(object sender, RoutedEventArgs e)
        {

        }

        private void m_searchBtn_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/SearchPage.xaml", UriKind.Relative));
        }

        private void m_homeBtn_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/HomePage.xaml", UriKind.Relative));
        }

        private void m_emergencyBtn2_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/EmergencyPage.xaml", UriKind.Relative));
        }

        private void m_helpBtn_Click(object sender, RoutedEventArgs e)
        {
            //TODO
        }

        private void m_applyBtn_Click(object sender, RoutedEventArgs e)
        {
            FrameworkElement root = Application.Current.RootVisual as FrameworkElement;
            CommsPage commsPage = root.DataContext as CommsPage;
            commsPage.AddNewBtn(m_previewBtn, m_description );


            NavigationService.GoBack();
        }
    }
}