﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using PowerDroid.TextToSpeech;
using System.IO.IsolatedStorage;
using System.IO;
using System.Device.Location;

namespace PowerDroid
{
    public partial class EnrichedContactsDetailPage : PhoneApplicationPage
    {
        private String m_contactNum = "111-111-1111";
        private LanguageServiceClient LanguageClient = new LanguageServiceClient();
        private WebClient WClient = new WebClient();
        GeoCoordinateWatcher m_watcher;
        private String m_locationString = "Unknown";

        public EnrichedContactsDetailPage()
        {
            InitializeComponent();
            powerAppBar1.parent = this;

            LanguageClient.SpeakCompleted += SpeakCompleted;
            WClient.OpenReadCompleted += WCOpenReadCompleted;

            if (m_watcher == null)
            {
                m_watcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High); // Use high accuracy.
                m_watcher.MovementThreshold = 20; // Use MovementThreshold to ignore noise in the signal.
                //m_watcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(watcher_StatusChanged);
                m_watcher.PositionChanged += this.watcher_PositionChanged;
                m_watcher.Start();

                //System.Diagnostics.Debug.WriteLine("m_watcher.Status => " + m_watcher.Status.ToString());
            }
         
        }

        void SpeakCompleted(object sender, SpeakCompletedEventArgs e)
        {
            if (e.Error != null)
                return;

            WClient.OpenReadAsync(new Uri(e.Result, UriKind.Absolute));
        }

        void WCOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            TextToSpeechMediaElement.Source = null;
            string temp = "tempTextToSpeech";
            using (IsolatedStorageFile userStoreForApplication = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (userStoreForApplication.FileExists(temp))
                    userStoreForApplication.DeleteFile(temp);

                var storageFileStream = userStoreForApplication.CreateFile(temp);

                using (storageFileStream)
                {
                    CopyStream(e.Result, storageFileStream);
                    if (e.Error == null)
                    {
                        TextToSpeechMediaElement.SetSource(storageFileStream);
                        TextToSpeechMediaElement.Play();
                    }
                }
            }
        }
        private void CopyStream(Stream input, Stream output)
        {
            byte[] buffer = new byte[32768];
            while (true)
            {
                int read = input.Read(buffer, 0, buffer.Length);
                if (read <= 0)
                    return;
                output.Write(buffer, 0, buffer.Length);
            }
        }

        private void PlaySentence(string sentence)
        {
            LanguageClient.SpeakAsync("A05A61388A448F55C9628C941FB2FC459D697060", sentence, "en", String.Empty);
        }

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            App globalApp = (Application.Current as App);
            this.DetailContactButton.Background = globalApp.CONTACT_IMG;
            this.DetailContactText.Text = globalApp.CONTACT_NAME;
            this.m_contactNum = globalApp.CONTACT_NUMBER;

        }

        private void SpeakFam1Button_Click(object sender, RoutedEventArgs e)
        {
            PlaySentence(DetailContactText.Text);
        }

        private void m_callBtn_Click(object sender, RoutedEventArgs e)
        {
            PhoneCallTask call = new PhoneCallTask();
            call.DisplayName = DetailContactText.Text;
            call.PhoneNumber = m_contactNum;
            call.Show();
        }

        private void m_txtBtn_Click(object sender, RoutedEventArgs e)
        {
            SmsComposeTask smsComposeTask = new SmsComposeTask();
            smsComposeTask.To = m_contactNum;
            smsComposeTask.Body = "";
            smsComposeTask.Show();
        }

        private void m_shareLocBtn_Click(object sender, RoutedEventArgs e)
        {
            SmsComposeTask smsComposeTask = new SmsComposeTask();
            smsComposeTask.To = m_contactNum;
            smsComposeTask.Body = "I'm located at: " + m_locationString;
            smsComposeTask.Show();
        }

        private void watcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            //var epl = e.Position.Location;

            // Access the position information thusly:
            //epl.Latitude.ToString("0.000");
            //epl.Longitude.ToString("0.000");
            //epl.Altitude.ToString();
            //epl.HorizontalAccuracy.ToString();
            //epl.VerticalAccuracy.ToString();
            //epl.Course.ToString();
            //epl.Speed.ToString();
            //e.Position.Timestamp.LocalDateTime.ToString();
            
            if (m_watcher.Status == GeoPositionStatus.Ready)
            {
                m_locationString = e.Position.Location.ToString();
            }
            else //emulator does not work with gps since no physical device
            {
                m_locationString = "Unknown";
            }

        }
    }
}